#!usr/bin/env python
"""
"""

import math
import os
import numpy as np
import pybullet as p


def prGreen(skk): print("\033[92m {}\033[00m" .format(skk))


def prBlue(skk): print("\033[94m {}\033[00m" .format(skk))


def prRed(skk): print("\033[91m {}\033[00m" .format(skk))


def render_environment(settings):
    cid = p.connect(p.SHARED_MEMORY)
    if (cid < 0):
        cid = p.connect(p.GUI)
    p.resetDebugVisualizerCamera(*settings)
    p.configureDebugVisualizer(p.COV_ENABLE_GUI, 0)


def create_velocity_sliders():
    motorsIds = []
    action = 1
    for param in ['posX', 'posY', 'posZ', 'rotX', 'rotY', 'rotZ']:
        motorsIds.append(p.addUserDebugParameter(param, -action, action, 0))
    return motorsIds


class Robotless():
    def __init__(self):
        '''
        Object wrapper to load a robotless .urdf file into pybullet.    
        '''

        self.base_pos = [0.0, 0.0, 0.0]
        self.base_orn = p.getQuaternionFromEuler([0, 0, 0])

        self.tool_link = 0
        self.max_force = -1

        self.id = p.loadURDF(os.path.join(os.getcwd(), 'end_effector.urdf'),
                             basePosition=self.base_pos,
                             baseOrientation=self.base_orn,
                             useFixedBase=False
                             )
        
        self.base_constraint = p.createConstraint(
            parentBodyUniqueId = self.id,
            parentLinkIndex = -1,
            childBodyUniqueId = -1,
            childLinkIndex = -1,
            jointType = p.JOINT_FIXED,
            jointAxis = [0, 0, 0],
            parentFramePosition = [0, 0, 0],
            childFramePosition = self.base_pos,
            childFrameOrientation = self.base_orn
        )

    def display_tool_frame(self, line_length=0.05):
        '''
        '''
        p.addUserDebugLine([0,0,0], [line_length,0,0], [1,0,0], parentObjectUniqueId=self.id, parentLinkIndex=self.tool_link)
        p.addUserDebugLine([0,0,0], [0,line_length,0], [0,1,0], parentObjectUniqueId=self.id, parentLinkIndex=self.tool_link)
        p.addUserDebugLine([0,0,0], [0,0,line_length], [0,0,1], parentObjectUniqueId=self.id, parentLinkIndex=self.tool_link)

    def apply_action(self, velocity, time_step):
        '''
        '''
        pos_lin = np.array(velocity[0:3]) * time_step
        pos_rot = np.array(velocity[3:6]) * time_step

        base_pos, base_orn = p.getBasePositionAndOrientation(self.id)
        new_pos = np.add(np.array(base_pos), pos_lin).tolist()
        new_orn = np.add(np.array(p.getEulerFromQuaternion(base_orn)), pos_rot)
        new_orn = p.getQuaternionFromEuler(new_orn.tolist())

        p.changeConstraint(self.base_constraint, new_pos, new_orn, maxForce=self.max_force)
